import numpy as np
import torch

from vlab.core import Backends, get_backend


def to_number(x):
    if isinstance(x, str):
        return float(x)
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return x.item()
    elif backend == Backends.TORCH:
        return x.detach().cpu().item()
    elif backend == Backends.BUILT_IN:
        return x
    else:
        raise TypeError(f"Unsupported backend {backend}")


def to_numpy(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY or backend == Backends.BUILT_IN:
        return np.asarray(x)
    elif backend == Backends.TORCH:
        return x.detach().cpu().numpy()
    else:
        raise TypeError(f"Unsupported backend {backend}")


def to_torch(x, dtype=None, device=None):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return torch.from_numpy(x).to(dtype=dtype, device=device)
    elif backend == Backends.TORCH:
        return x
    elif backend == Backends.BUILT_IN:
        return torch.tensor(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def to_ref(x, ref):
    backend = get_backend(ref)
    if backend == Backends.NUMPY:
        return to_numpy(x)
    elif backend == Backends.TORCH:
        return to_torch(x, dtype=ref.dtype, device=ref.device)
    elif backend == Backends.BUILT_IN:
        return to_number(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")
