import numpy as np
import torch

from vlab.core import Backends, get_backend


def eye(n, ref):
    backend = get_backend(ref)
    if backend == Backends.NUMPY:
        return np.eye(n, dtype=ref.dtype)
    elif backend == Backends.TORCH:
        return torch.eye(n, dtype=ref.dtype, device=ref.device)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def ones(shape, ref):
    backend = get_backend(ref)
    if backend == Backends.NUMPY:
        return np.ones(shape, dtype=ref.dtype)
    elif backend == Backends.TORCH:
        return torch.ones(shape, dtype=ref.dtype, device=ref.device)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def empty(shape, ref):
    backend = get_backend(ref)
    if backend == Backends.NUMPY:
        return np.empty(shape, dtype=ref.dtype)
    elif backend == Backends.TORCH:
        return torch.empty(shape, dtype=ref.dtype, device=ref.device)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def zeros(shape, ref):
    backend = get_backend(ref)
    if backend == Backends.NUMPY:
        return np.zeros(shape, dtype=ref.dtype)
    elif backend == Backends.TORCH:
        return torch.zeros(shape, dtype=ref.dtype, device=ref.device)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def tensor(data, ref):
    backend = get_backend(ref)
    if backend == Backends.NUMPY:
        return np.array(data, dtype=ref.dtype)
    elif backend == Backends.TORCH:
        return torch.tensor(data, dtype=ref.dtype, device=ref.device)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def ones_like(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.ones_like(x)
    elif backend == Backends.TORCH:
        return torch.ones_like(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def zeros_like(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.zeros_like(x)
    elif backend == Backends.TORCH:
        return torch.zeros_like(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def empty_like(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.empty_like(x)
    elif backend == Backends.TORCH:
        return torch.empty_like(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def as_tensor(x):
    if isinstance(x, list) or isinstance(x, tuple):
        backend = Backends.NUMPY
    else:
        backend = get_backend(x)

    if backend == Backends.NUMPY or backend == Backends.BUILT_IN:
        return np.asarray(x)
    elif backend == Backends.TORCH:
        return torch.as_tensor(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")
