import numpy as np
import torch

from vlab.core import Backends, get_backend


def sqrt(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.sqrt(x)
    elif backend == Backends.TORCH:
        return torch.sqrt(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def abs(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.abs(x)
    elif backend == Backends.TORCH:
        return torch.abs(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def sin(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.sin(x)
    elif backend == Backends.TORCH:
        return torch.sin(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def asin(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.arcsin(x)
    elif backend == Backends.TORCH:
        return torch.asin(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def acos(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.arccos(x)
    elif backend == Backends.TORCH:
        return torch.acos(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def cos(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.cos(x)
    elif backend == Backends.TORCH:
        return torch.cos(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def tan(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.tan(x)
    elif backend == Backends.TORCH:
        return torch.tan(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def atan(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.arctan(x)
    elif backend == Backends.TORCH:
        return torch.atan(x)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def atan2(input, other):
    backend = get_backend(input)
    if backend == Backends.NUMPY:
        return np.arctan2(input, other)
    elif backend == Backends.TORCH:
        return torch.atan2(input, other)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def isclose(x1, x2, rtol=1e-05, atol=1e-08):
    backend = get_backend(x1)
    backend = get_backend(x2) if backend == Backends.BUILT_IN else backend
    if backend == Backends.NUMPY:
        return np.isclose(x1, x2, rtol=rtol, atol=atol)
    elif backend == Backends.TORCH:
        return torch.isclose(x1, x2, rtol=rtol, atol=atol)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def maximum(x1, x2):
    backend = get_backend(x1)
    backend = get_backend(x2) if backend == Backends.BUILT_IN else backend
    if backend == Backends.NUMPY:
        return np.maximum(x1, x2)
    elif backend == Backends.TORCH:
        return torch.maximum(x1, x2)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def minimum(x1, x2):
    backend = get_backend(x1)
    backend = get_backend(x2) if backend == Backends.BUILT_IN else backend
    if backend == Backends.NUMPY:
        return np.minimum(x1, x2)
    elif backend == Backends.TORCH:
        return torch.minimum(x1, x2)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def clamp(x, min_value=None, max_value=None):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.clip(x, min_value, max_value)
    elif backend == Backends.TORCH:
        return torch.clamp(x, min_value, max_value)
    else:
        raise TypeError(f"Unsupported backend {backend}")
