import numpy as np
import torch
import torch.nn.functional as F

from vlab.core import Backends, get_backend


def clone(x):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return x.copy()
    elif backend == Backends.TORCH:
        return x.clone()
    else:
        raise TypeError(f"Unsupported backend {backend}")


def unbind(x, dim):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        x_split = np.split(x, x.shape[dim], dim)
        return [np.squeeze(item, dim) for item in x_split]
    elif backend == Backends.TORCH:
        return torch.unbind(x, dim=dim)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def one_hot(indices, num_classes):
    backend = get_backend(indices)
    if backend == Backends.NUMPY:
        # from https://stackoverflow.com/questions/38592324/one-hot-encoding-using-numpy
        res = np.eye(num_classes)[np.array(indices).reshape(-1)]
        return res.reshape(list(indices.shape) + [num_classes])
    elif backend == Backends.TORCH:
        return F.one_hot(indices, num_classes)
    else:
        raise TypeError(f"Unsupported backend {backend}")
