import numpy as np
import torch

from vlab.core import Backends, get_backend


def stack(tensors, dim=0):
    backend = get_backend(tensors[0])
    if backend == Backends.NUMPY:
        return np.stack(tensors, axis=dim)
    elif backend == Backends.TORCH:
        return torch.stack(tensors, dim=dim)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def cat(tensors, dim=0):
    backend = get_backend(tensors[0])
    if backend == Backends.NUMPY:
        return np.concatenate(tensors, axis=dim)
    elif backend == Backends.TORCH:
        return torch.cat(tensors, dim=dim)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def transpose(x, dim0, dim1):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.swapaxes(x, axis1=dim0, axis2=dim1)
    elif backend == Backends.TORCH:
        return torch.transpose(x, dim0=dim0, dim1=dim1)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def repeat(x, reps):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.tile(x, reps=reps)
    elif backend == Backends.TORCH:
        return x.repeat(reps)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def broadcast_to(x, size):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.broadcast_to(x, size)
    elif backend == Backends.TORCH:
        return torch.broadcast_to(x, size)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def squeeze(x, dim):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.squeeze(x, axis=dim)
    elif backend == Backends.TORCH:
        return torch.squeeze(x, dim=dim)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def unsqueeze(x, dim):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.expand_dims(x, axis=dim)
    elif backend == Backends.TORCH:
        return torch.unsqueeze(x, dim=dim)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def reshape(x, shape):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.reshape(x, shape)
    elif backend == Backends.TORCH:
        return torch.reshape(x, shape)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def permute(x, dims):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        return np.transpose(x, axes=dims)
    elif backend == Backends.TORCH:
        return torch.permute(x, dims=dims)
    else:
        raise TypeError(f"Unsupported backend {backend}")
