import numpy as np
import torch

from vlab.core import Backends, get_backend


def where(condition, input=None, other=None):
    backend = get_backend(condition)
    if backend == Backends.NUMPY:
        return np.where(condition, input, other)
    elif backend == Backends.TORCH:
        if input is None or other is None:
            raise ValueError("Both input and other must be provided for torch backend")
        return torch.where(condition, input, other)
    else:
        raise TypeError(f"Unsupported backend {backend}")


def argmax(x, dim=None, keepdim=False):
    backend = get_backend(x)
    if backend == Backends.NUMPY:
        if keepdim:
            raise NotImplementedError("keepdim=True is not supported for numpy backend")
        return np.argmax(x, axis=dim)
    elif backend == Backends.TORCH:
        return torch.argmax(x, dim=dim, keepdim=keepdim)
    else:
        raise TypeError(f"Unsupported backend {backend}")
