import os
from pathlib import PurePath

import cv2


def imread(filepath, to_rgb=True):
    if isinstance(filepath, PurePath):
        filepath = str(filepath)

    if not os.path.exists(filepath):
        raise FileNotFoundError(filepath)  # cv2 error is so cryptic

    img = cv2.imread(filepath, cv2.IMREAD_ANYDEPTH | cv2.IMREAD_UNCHANGED)

    if to_rgb and img.ndim == 3:
        if img.shape[-1] == 3:
            img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        elif img.shape[-1] == 4:
            img = cv2.cvtColor(img, cv2.COLOR_BGRA2RGBA)

    return img


def imwrite(img, filepath, from_rgb=True, auto_mkdirs=False):
    if isinstance(filepath, PurePath):
        filepath = str(filepath)

    if from_rgb and img.ndim == 3:
        if img.shape[-1] == 3:
            img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
        elif img.shape[-1] == 4:
            img = cv2.cvtColor(img, cv2.COLOR_RGBA2BGRA)

    if auto_mkdirs:
        os.makedirs(os.path.dirname(filepath), exist_ok=True)

    cv2.imwrite(filepath, img)
