import numpy as np

from vlab.core.general import Backends, get_backend


def imshow(img, title=None, dpi=200, backend="plt"):
    from matplotlib import pyplot as plt

    # handle torch tensor
    if get_backend(img) == Backends.TORCH:
        img = img.detach().cpu().numpy()

    # handle batch size
    if img.ndim == 4 and img.shape[0] == 1:
        img = img[0]

    # handle [CHW] order
    if img.ndim == 3 and img.shape[0] == 3 and img.shape[-1] != 3:
        img = np.transpose(img, (1, 2, 0))

    if title is not None:
        plt.title(str(title))
    plt.imshow(img)
    plt.gcf().set_dpi(dpi)
    plt.axis("off")
    plt.tight_layout()
    plt.show()
